#!/bin/bash -e
# This script tests the base image and invokes the secure baseline audit
# If any issues occur, it exits with non-zero exit code
VERBOSE=false
source /usr/local/bin/shared-baseimage.sh

function verboseLog {
    if $VERBOSE; then
        echo "INFO: $@"
    fi
}

function bailIfFileMissingVerbose {
    file="$@"
    bailIfFileMissing "$file"
    verboseLog "File [$file] exists"
}

while (( "$#" )); do
    if [ "$1" == "-v" ]; then
        VERBOSE=true
    fi
    shift
done

# Using 2.x azure-security package
AZSEC_DAEMON=azsecd
AZSEC_CONFDIR=/etc/azsec/scans.d

# BASE IMAGE TESTS
mdsd -V || bail "Error running mdsd -V. Build will now abort."
mdsd -v || bail "Error validating valid MDSD config. Build will now abort."
verboseLog "Ran mdsd -V"

# Wait up to 60 seocnds for mdsd to start
service mdsd start
startTime=$(date +%s)
maxTime=60
while true; do
    elapsed=$(($(date +%s)-$startTime))
    if netstat -lnt | grep -q "127\.0\.0\.1:29130"; then
        echo "Running mdsd instance found. Continuing."
        break;
    elif [ $elapsed -ge $maxTime ]; then
        echo "No mdsd instance found after $maxTime. Exiting."
        exit 1
    else
        echo "No mdsd instance found after $elapsed seconds. Waiting..."
        sleep 5
    fi
done

# Don't care if this generates an error, we care if logs are generated
bailIfFileMissingVerbose /var/log/mdsd.err

service azsecd start || echo Service is already started - continuing
ps -C $AZSEC_DAEMON || bail "Azure security scanning service not started"
verboseLog "$AZSEC_DAEMON is running"
disabledScans=$(grep -L cycle=\"P $AZSEC_CONFDIR/*)
if [ -n "$disabledScans" ]; then
    bail "Scans are not enabled in scan file $disabledScans"
else
    verboseLog "azsecd scans are enabled"
fi

fqdn=$(hostname -f) || bail "Error running hostname -f. Build will now abort."

echo "FQDN = $fqdn"
cloudName=$(GetCloudName)
if [ -z "$cloudName" ]; then
    bail "Hostname -f [$fqdn] does not match any expected DNS suffix"
else
    verboseLog "FQDN $fqdn matches $cloudName"
fi

if [[ "${OSMAJORVER}" -lt 18 ]]; then
    grep 'options timeout:1 attempts:5' /etc/resolv.conf || bail "Retry options not found in /etc/resolv.conf"
    egrep '^hosts:\s+files\s+cache' /etc/nsswitch.conf || bail "unscd not enabled in /etc/nsswitch.conf"
    egrep '^\s*enable-cache\s+hosts\s+yes' /etc/nscd.conf || bail "Host caching not enabled in /etc/nscd.conf"

    source /etc/profile
    if [ -z "$JAVA_HOME" ] || [ ! -d $JAVA_HOME ]; then
        bail "JAVA_HOME [$JAVA_HOME] is not a valid directory. Build will now abort"
    else
        verboseLog "JAVA_HOME is a valid directory [$JAVA_HOME]"
    fi
fi

ofilelimit=$(egrep -m 1 '^\*\s+hard\s+nofile' /etc/security/limits.conf | awk '{print $4}')
expectedfilelimit=10000
if [ -z "$ofilelimit" ]; then
    bail "No open file limit specified in /etc/security/limits.conf. Please specify a value of 10000 (nofile 10000)"
elif [ $ofilelimit -lt $expectedfilelimit ]; then
    bail "File limit [$ofilelimit] should be $expectedfilelimit"
else
    verboseLog "File limit [$ofilelimit] is >= $expectedfilelimit"
fi

pamLimitCount=$(egrep -c "^session\s+required\s+pam_limits.so" /etc/pam.d/common-session)
if [ $pamLimitCount -lt 1 ]; then
    bail "No lines found in /etc/pam.d/common-session that match 'session required pam_limits.so'"
else
    verboseLog "Session limits enabled in /etc/pam.d/common-session"
fi

# Patching tests
bailIfFileMissingVerbose /etc/cron.daily/sbi-selfpatch
unattendScript=/usr/bin/unattended-upgrade
if ! grep -q "^def is_update_day" $unattendScript; then
    bail "$unattendScript does not contain 'is_update_day' function"
else
    echo "$unattendScript supports day-of-week patching"
fi

# Dhclient regression test
if IsDhcpOptionBroken; then
    dhcpOption=$(grep "send host-name" $dhclientConf)
    bail "DHCP option [$dhcpOption] is not valid for Ubuntu $OSVER"
fi

# SECURE BASELINE TESTS
ovfEnvXml=/var/lib/waagent/ovf-env.xml
plaintextPass=$(egrep -c 'UserPassword>REDACTED|PublicKey' $ovfEnvXml)
if [ $plaintextPass -lt 1 ] ; then
    bail "Password was not redacted from $ovfEnvXml"
else
    verboseLog "Plaintext password not found in $ovfEnvXml"
fi

# Confirm we comply with the baseline
# Parse a scan result file in /var/lib/azsec
rm -f /var/lib/azsec/*
$AZSEC_DAEMON manual -s baseline
scanFile=$(ls /var/lib/azsec/baseline* | tail -n 1)
results=$(cat $scanFile)
OIFS=$IFS
IFS=$'\n'
if ! echo $results | grep -q "0 errors and 0 findings"; then
    # Loop through failures. If any of the failures doesn't match our permitted list, bail
    failCount=0
    for element in $( echo $results | sed 's/|@|/\n/g' | awk '/FAIL/ {print $1}' ); do
        if [[ "$element" =~ ^7= ]];then
            echo "OK to have some missing patches. Ignoring..."
        elif [ $OSMAJORVER -ne 14 ] && [[ "$element" =~ ^36= ]]; then
            echo "x11-common required for JDK in 12.04 and JRE8 in 16.04 image. Ignoring..."
        elif [[ "$element" =~ ^106\.11= ]]; then
            echo "Restricting OpenSSH users is only plausible for a real deployment. Ignoring..."
        elif [ $OSMAJORVER -eq 14 ] && [[ "$element" =~ ^181= ]]; then
            echo "rsync service is deactivated by default, but the policy check is based on systemd and will fail. Ignoring..."
        else
            # This finding isn't one we can ignore. Fail the build.
            echo $element
            failCount=$((failCount+1))
        fi
    done
    if [ $failCount -gt 0 ]; then
        bail "Baseline security scan failed [$failCount failures]"
    fi
else
    verboseLog "Baseline scan completed successfully"
fi
IFS=$OIFS

# Confirm the artifacts for the unlock accounts script are present
repairBaseName=repair-baseimage
bailIfFileMissingVerbose "/etc/cron.d/$repairBaseName"
bailIfFileMissingVerbose "/usr/local/sbin/$repairBaseName.sh"

# CLAMAV TEST
bailIfFileMissingVerbose "/var/lib/azsec-clamav/main.cvd"

service mdsd stop
echo "SUCCESS! All tests passed"
